@@ -35,14 +35,19 @@
 #include <QSocketNotifier>
 
 //policykit header
-#include <polkit-dbus/polkit-dbus.h>
-#include <polkit-grant/polkit-grant.h>
+// #include <polkit-dbus/polkit-dbus.h>
+
+#include <Context>
 
 #include "AuthDialog.h"
 #include "processwatcher.h"
 
 #define THIRTY_SECONDS 30000
 
+using namespace PolkitQt;
+
+KCONFIGGROUP_DECLARE_ENUM_QOBJECT(PolicyKitKDE,KeepPassword)
+
 PolicyKitKDE *PolicyKitKDE::m_self;
 
 //----------------------------------------------------------------------------
@@ -71,25 +76,13 @@ PolicyKitKDE::PolicyKitKDE()
     }
 
     m_context = polkit_context_new();
-    if (m_context == NULL) {
-        kDebug() << "Could not get a new PolKitContext.";
-        QTimer::singleShot(0, this, SLOT(quit()));
-        return;
-    }
-
     polkit_context_set_load_descriptions(m_context);
+    // we must pass the created context with the
+    // polkit_context_set_load_descriptions() before the
+    // lib inits it.
+    Context::instance(m_context);
 
-    polkit_context_set_config_changed(m_context, polkit_config_changed, this);
-    polkit_context_set_io_watch_functions(m_context, pk_io_add_watch, pk_io_remove_watch);
-
-    PolKitError* error = NULL;
-    if (!polkit_context_init(m_context, &error)) {
-        QString msg("Could not initialize PolKitContext");
-        if (polkit_error_is_set(error)) {
-            kError() << msg <<  ": " << polkit_error_get_error_message(error);
-            polkit_error_free(error);
-        } else
-            kError() << msg;
+    if (Context::instance()->hasError()) {
         QTimer::singleShot(0, this, SLOT(quit()));
         return;
     }
@@ -123,14 +116,14 @@ bool PolicyKitKDE::ObtainAuthorization(const QString &actionId, uint wid, uint p
     inProgress = true;
     m_gainedPrivilege = false;
     m_requiresAdmin = false;
-    keepPassword = KeepPasswordNo;
+    m_keepPassword = KeepPasswordNo;
 
-    action = polkit_action_new();
-    if (action == NULL) {
+    m_pkAction = polkit_action_new();
+    if (m_pkAction == NULL) {
         kError() << "Could not create new polkit action.";
         return false;
     }
-    if (!polkit_action_set_action_id(action, actionId.toLatin1())) {
+    if (!polkit_action_set_action_id(m_pkAction, actionId.toLatin1())) {
         kError() << "Could not set actionid.";
         return false;
     }
@@ -155,7 +148,7 @@ bool PolicyKitKDE::ObtainAuthorization(const QString &actionId, uint wid, uint p
     }
 
     kDebug() << "Getting policy cache entry for an action...";
-    PolKitPolicyFileEntry *entry = polkit_policy_cache_get_entry(cache, action);
+    PolKitPolicyFileEntry *entry = polkit_policy_cache_get_entry(cache, m_pkAction);
     if (entry == NULL) {
         kWarning() << "Could not get policy entry for action.";
         return false;
@@ -172,15 +165,17 @@ bool PolicyKitKDE::ObtainAuthorization(const QString &actionId, uint wid, uint p
     connect(dialog, SIGNAL(adminUserSelected(QString)), SLOT(userSelected(QString)));
     if (wid != 0) {
         KWindowSystem::setMainWindow(dialog, wid);
-    } else
+    } else {
         updateUserTimestamp(); // make it get focus unconditionally :-/
+    }
 
     parent_wid = wid;
 
     message().setDelayedReply(true);
     reply = message().createReply();
 
-    QDBusConnection::sessionBus().send(reply);
+    // Needed to don't need to return imediately
+    setDelayedReply(true);
 
     m_killT->stop();
     m_numTries = 0;
@@ -208,7 +203,7 @@ void PolicyKitKDE::tryAgain()
     m_wasBogus = false;
     m_newUserSelected = false;
 
-    if (!polkit_grant_initiate_auth(grant, action, caller)) {
+    if (!polkit_grant_initiate_auth(grant, m_pkAction, caller)) {
         kWarning() << "Failed to initiate privilege grant.";
         // send the reply over D-Bus:
         reply << true;
@@ -217,6 +212,9 @@ void PolicyKitKDE::tryAgain()
     }
 }
 
+
+
+
 void PolicyKitKDE::finishObtainPrivilege()
 {
     if (m_newUserSelected) {
@@ -244,14 +242,13 @@ void PolicyKitKDE::finishObtainPrivilege()
     }
 
     if (m_gainedPrivilege) {
-        /* add to blacklist if the user unchecked the "remember authorization" check box */
-        //TODO store the user's preference
-//             if ((ud->remember_always &&
-//                     !polkit_gnome_auth_dialog_get_remember_always (POLKIT_GNOME_AUTH_DIALOG (ud->dialog))) ||
-//                 (ud->remember_session &&
-//                     !polkit_gnome_auth_dialog_get_remember_session (POLKIT_GNOME_AUTH_DIALOG (ud->dialog)))) {
-//                     add_to_blacklist (ud, action_id);
-//             }
+        // add to blacklist if the user unchecked the "remember authorization" check box
+        char *action_id;
+        polkit_action_get_action_id(m_pkAction, &action_id);
+        // here we store the action preference
+        KConfig config;
+        KConfigGroup actionsPreferences(&config, "ActionsPreferences");
+        writeEntry(actionsPreferences, action_id, m_keepPassword);
     }
 
     // send the reply over D-Bus:
@@ -266,12 +263,15 @@ void PolicyKitKDE::finishObtainPrivilege()
     m_adminUsers.clear();
     m_adminUserSelected.clear();
 
-    if (grant != NULL)
+    if (grant != NULL) {
         polkit_grant_unref(grant);
-    if (action != NULL)
-        polkit_action_unref(action);
-    if (caller != NULL)
+    }
+    if (m_pkAction != NULL) {
+        polkit_action_unref(m_pkAction);
+    }
+    if (caller != NULL) {
         polkit_caller_unref(caller);
+    }
 
     grant = NULL;
 
@@ -280,40 +280,47 @@ void PolicyKitKDE::finishObtainPrivilege()
     kDebug() << "Finish obtain authorization:" << m_gainedPrivilege;
 }
 
+PolicyKitKDE::KeepPassword PolicyKitKDE::readDefaultKeepPassword(const QString &actionId, const KeepPassword defaultValue)
+{
+    KConfig config;
+    KConfigGroup actionsPreferences(&config, "ActionsPreferences");
+    return readEntry(actionsPreferences, actionId.toLatin1(), defaultValue);
+}
+
 void PolicyKitKDE::conversation_type(PolKitGrant *grant, PolKitResult type, void *user_data)
 {
     PolicyKitKDE *self = (PolicyKitKDE *) user_data;
     kDebug() << "conversation_type" << grant << type;
 
     self->m_requiresAdmin = false;
-    self->keepPassword = KeepPasswordNo;
+    self->m_keepPassword = KeepPasswordNo;
     switch (type) {
-    case POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_ONE_SHOT:
-    case POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH:
-        self->m_requiresAdmin = true;
-        break;
-    case POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_KEEP_SESSION:
-        self->m_requiresAdmin = true;
-        self->keepPassword = KeepPasswordSession;
-        break;
-    case POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_KEEP_ALWAYS:
-        self->m_requiresAdmin = true;
-        self->keepPassword = KeepPasswordAlways;
-        break;
-    case POLKIT_RESULT_ONLY_VIA_SELF_AUTH_ONE_SHOT:
-    case POLKIT_RESULT_ONLY_VIA_SELF_AUTH:
-        break;
-    case POLKIT_RESULT_ONLY_VIA_SELF_AUTH_KEEP_SESSION:
-        self->keepPassword = KeepPasswordSession;
-        break;
-    case POLKIT_RESULT_ONLY_VIA_SELF_AUTH_KEEP_ALWAYS:
-        self->keepPassword = KeepPasswordAlways;
-        break;
-    default:
-        abort();
+        case POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_ONE_SHOT:
+        case POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH:
+            self->m_requiresAdmin = true;
+            break;
+        case POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_KEEP_SESSION:
+            self->m_requiresAdmin = true;
+            self->m_keepPassword = KeepPasswordSession;
+            break;
+        case POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_KEEP_ALWAYS:
+            self->m_requiresAdmin = true;
+            self->m_keepPassword = KeepPasswordAlways;
+            break;
+        case POLKIT_RESULT_ONLY_VIA_SELF_AUTH_ONE_SHOT:
+        case POLKIT_RESULT_ONLY_VIA_SELF_AUTH:
+            break;
+        case POLKIT_RESULT_ONLY_VIA_SELF_AUTH_KEEP_SESSION:
+            self->m_keepPassword = KeepPasswordSession;
+            break;
+        case POLKIT_RESULT_ONLY_VIA_SELF_AUTH_KEEP_ALWAYS:
+            self->m_keepPassword = KeepPasswordAlways;
+            break;
+        default:
+            abort();
     }
 
-    self->dialog->setOptions(self->keepPassword, self->m_requiresAdmin, self->m_adminUsers);
+    self->dialog->setOptions(self->m_keepPassword, self->m_requiresAdmin, self->m_adminUsers);
 }
 
 char* PolicyKitKDE::conversation_select_admin_user(PolKitGrant *polkit_grant, char **admin_users, void *user_data)
@@ -325,15 +332,16 @@ char* PolicyKitKDE::conversation_select_admin_user(PolKitGrant *polkit_grant, ch
     /* if we've already selected the admin user.. then reuse the same one (this
      * is mainly when the user entered the wrong password)
      */
-    if (!(currentAdminUser = self->dialog->adminUserSelected()).isEmpty())
+    if (!(currentAdminUser = self->dialog->adminUserSelected()).isEmpty()) {
         return strdup(currentAdminUser.toLocal8Bit());
+    }
 
     QStringList adminUsers;
     for (int i = 0; admin_users[i] != NULL; i++) {
         adminUsers << admin_users[i];
     }
     self->m_adminUsers = adminUsers;
-    self->dialog->setOptions(self->keepPassword, self->m_requiresAdmin, self->m_adminUsers);
+    self->dialog->setOptions(self->m_keepPassword, self->m_requiresAdmin, self->m_adminUsers);
 
     // if we are running as one of the users in adminUsers then preselect that user...
     if (!(currentAdminUser = self->dialog->selectCurrentAdminUser()).isEmpty()) {
@@ -362,7 +370,7 @@ char* PolicyKitKDE::conversation_select_admin_user(PolKitGrant *polkit_grant, ch
 
 void PolicyKitKDE::dialogAccepted()
 {
-    keepPassword = dialog->keepPassword();
+    m_keepPassword = dialog->keepPassword();
     kDebug() << "Password dialog confirmed.";
 }
 
@@ -370,7 +378,6 @@ void PolicyKitKDE::dialogCancelled()
 {
     m_wasCancelled = true;
     kDebug() << "Password dialog cancelled.";
-//     polkit_grant_cancel_auth(grant);
 }
 
 char* PolicyKitKDE::conversation_pam_prompt(PolKitGrant *polkit_grant, const char *request, void *user_data, bool echoOn)
@@ -379,7 +386,7 @@ char* PolicyKitKDE::conversation_pam_prompt(PolKitGrant *polkit_grant, const cha
     PolicyKitKDE *self = (PolicyKitKDE *) user_data;
     kDebug() << QString("request=%1, echo_on=%2").arg(request).arg(echoOn);
     self->dialog->setRequest(request, self->m_requiresAdmin);
-    self->dialog->setOptions(self->keepPassword, self->m_requiresAdmin, self->m_adminUsers);
+    self->dialog->setOptions(self->m_keepPassword, self->m_requiresAdmin, self->m_adminUsers);
     self->dialog->setPasswordShowChars(echoOn);
     self->dialog->show();
 
@@ -450,63 +457,65 @@ PolKitResult PolicyKitKDE::conversation_override_grant_type(PolKitGrant *polkit_
     bool keep_always = false;
 
     switch (type) {
-    case POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_ONE_SHOT:
-    case POLKIT_RESULT_ONLY_VIA_SELF_AUTH_ONE_SHOT:
-    case POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH:
-    case POLKIT_RESULT_ONLY_VIA_SELF_AUTH:
-        break;
-    case POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_KEEP_SESSION:
-    case POLKIT_RESULT_ONLY_VIA_SELF_AUTH_KEEP_SESSION:
-        if (self->keepPassword == KeepPasswordSession)
-            keep_session = true;
-        break;
-    case POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_KEEP_ALWAYS:
-    case POLKIT_RESULT_ONLY_VIA_SELF_AUTH_KEEP_ALWAYS:
-        if (self->keepPassword == KeepPasswordAlways)
-            keep_always = true;
-        else if (self->keepPassword == KeepPasswordSession)
-            keep_session = true;
-        break;
-    default:
-        abort();
+        case POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_ONE_SHOT:
+        case POLKIT_RESULT_ONLY_VIA_SELF_AUTH_ONE_SHOT:
+        case POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH:
+        case POLKIT_RESULT_ONLY_VIA_SELF_AUTH:
+            break;
+        case POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_KEEP_SESSION:
+        case POLKIT_RESULT_ONLY_VIA_SELF_AUTH_KEEP_SESSION:
+            if (self->m_keepPassword == KeepPasswordSession)
+                keep_session = true;
+            break;
+        case POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_KEEP_ALWAYS:
+        case POLKIT_RESULT_ONLY_VIA_SELF_AUTH_KEEP_ALWAYS:
+            if (self->m_keepPassword == KeepPasswordAlways)
+                keep_always = true;
+            else if (self->m_keepPassword == KeepPasswordSession)
+                keep_session = true;
+            break;
+        default:
+            abort();
     }
     kDebug() << "Keep password, always:" << keep_always << ", session:" << keep_session;
     PolKitResult ret;
     switch (type) {
-    case POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_ONE_SHOT:
-        ret = POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_ONE_SHOT;
-        break;
-    case POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH:
-    case POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_KEEP_SESSION:
-    case POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_KEEP_ALWAYS:
-        if (keep_session)
-            ret = POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_KEEP_SESSION;
-        else if (keep_always)
-            ret = POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_KEEP_ALWAYS;
-        else
-            ret = POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH;
-        break;
-    case POLKIT_RESULT_ONLY_VIA_SELF_AUTH_ONE_SHOT:
-        ret = POLKIT_RESULT_ONLY_VIA_SELF_AUTH_ONE_SHOT;
-        break;
-    case POLKIT_RESULT_ONLY_VIA_SELF_AUTH:
-    case POLKIT_RESULT_ONLY_VIA_SELF_AUTH_KEEP_SESSION:
-    case POLKIT_RESULT_ONLY_VIA_SELF_AUTH_KEEP_ALWAYS:
-        if (keep_session)
-            ret = POLKIT_RESULT_ONLY_VIA_SELF_AUTH_KEEP_SESSION;
-        else if (keep_always)
-            ret = POLKIT_RESULT_ONLY_VIA_SELF_AUTH_KEEP_ALWAYS;
-        else
-            ret = POLKIT_RESULT_ONLY_VIA_SELF_AUTH;
-        break;
-    default:
-        abort();
+        case POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_ONE_SHOT:
+            ret = POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_ONE_SHOT;
+            break;
+        case POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH:
+        case POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_KEEP_SESSION:
+        case POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_KEEP_ALWAYS:
+            if (keep_session)
+                ret = POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_KEEP_SESSION;
+            else if (keep_always)
+                ret = POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_KEEP_ALWAYS;
+            else
+                ret = POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH;
+            break;
+        case POLKIT_RESULT_ONLY_VIA_SELF_AUTH_ONE_SHOT:
+            ret = POLKIT_RESULT_ONLY_VIA_SELF_AUTH_ONE_SHOT;
+            break;
+        case POLKIT_RESULT_ONLY_VIA_SELF_AUTH:
+        case POLKIT_RESULT_ONLY_VIA_SELF_AUTH_KEEP_SESSION:
+        case POLKIT_RESULT_ONLY_VIA_SELF_AUTH_KEEP_ALWAYS:
+            if (keep_session)
+                ret = POLKIT_RESULT_ONLY_VIA_SELF_AUTH_KEEP_SESSION;
+            else if (keep_always)
+                ret = POLKIT_RESULT_ONLY_VIA_SELF_AUTH_KEEP_ALWAYS;
+            else
+                ret = POLKIT_RESULT_ONLY_VIA_SELF_AUTH;
+            break;
+        default:
+            abort();
     }
     return ret;
 }
 
-void PolicyKitKDE::conversation_done(PolKitGrant *polkit_grant, polkit_bool_t gainedPrivilege,
-                                     polkit_bool_t inputWasBogus, void *user_data)
+void PolicyKitKDE::conversation_done(PolKitGrant *polkit_grant,
+                                     polkit_bool_t gainedPrivilege,
+                                     polkit_bool_t inputWasBogus,
+                                     void *user_data)
 {
     Q_UNUSED(polkit_grant);
     PolicyKitKDE *self = (PolicyKitKDE *) user_data;
@@ -578,35 +587,6 @@ void PolicyKitKDE::watchActivatedContext(int fd)
 
 //----------------------------------------------------------------------------
 
-int PolicyKitKDE::pk_io_add_watch(PolKitContext* context, int fd)
-{
-    kDebug() << "add_watch" << context << fd;
-
-    QSocketNotifier *notify = new QSocketNotifier(fd, QSocketNotifier::Read, m_self);
-    m_self->m_watches[fd] = notify;
-
-    notify->connect(notify, SIGNAL(activated(int)), m_self, SLOT(watchActivatedContext(int)));
-
-    return fd; // use simply the fd as the unique id for the watch
-}
-
-
-//----------------------------------------------------------------------------
-
-void PolicyKitKDE::pk_io_remove_watch(PolKitContext* context, int id)
-{
-//     assert(id > 0);
-    kDebug() << "remove_watch" << context << id;
-    if (!m_self->m_watches.contains(id))
-        return; // policykit likes to do this more than once
-
-    QSocketNotifier* notify = m_self->m_watches.take(id);
-    notify->deleteLater();
-    notify->setEnabled(false);
-}
-
-//----------------------------------------------------------------------------
-
 int PolicyKitKDE::add_child_watch(PolKitGrant*, pid_t pid)
 {
     ProcessWatch *watch = new ProcessWatch(pid);
@@ -619,7 +599,7 @@ int PolicyKitKDE::add_child_watch(PolKitGrant*, pid_t pid)
 
 void PolicyKitKDE::remove_child_watch(PolKitGrant*, int id)
 {
-//     assert(id < 0);
+    Q_ASSERT(id < 0);
     ProcessWatcher::instance()->remove(-id);
 }
 
@@ -634,16 +614,10 @@ void PolicyKitKDE::childTerminated(pid_t pid, int exitStatus)
 
 void PolicyKitKDE::remove_watch(PolKitGrant* grant, int id)
 {
-    if (id > 0)  // io watches are +, child watches are -
+    // io watches are +, child watches are -
+    if (id > 0) {
         remove_grant_io_watch(grant, id);
-    else
+    } else {
         remove_child_watch(grant, id);
-}
-
-//----------------------------------------------------------------------------
-void PolicyKitKDE::polkit_config_changed(PolKitContext *context, void *user_data)
-{
-    Q_UNUSED(context);
-    Q_UNUSED(user_data);
-    kDebug() << "PolicyKit reports that the config have changed";
+    }
 }
